/* src/admin_cmds.c - Diverses commandes pour admins
 * Copyright (C) 2002 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <main.h>

CVSINFO("$Id: admin_cmds.c,v 1.4 2002/11/17 17:08:34 coderz Exp $");

void inviteme(aNick *nick, int parc, char **parv)
{
    putserv(":%s INVITE %s %s\n", parv[0], nick->nick, uw.chan);
    return;
}

/*
 * do_admin
 *
 * parv[1] = pseudo
 * parv[2] = niveau
 */
void do_admin(aNick *nick, int parc, char **parv)
{
	anUser *user;
	int niveau;
	char passwd[20], p[15];

	if(parc<2)
	{
		syntax_cmd(uw.nick,nick->numeric,"ADDADMIN");
		return;
	}
	if(!is_num(parv[3]))
	{
		uwntc(nick->numeric, "Le niveau doit tre une valeur numrique");
		return;
	}
	niveau = atoi(parv[3]);
	if(niveau > 4 || niveau < 1)
	{
		uwntc(nick->numeric, "Le niveau des admins doit tre entre 1 et 4.");
		return;
	}
	if((user = getuserinfo(parv[1])))
	{
		uwntc(nick->numeric, "%s est dj un username enregistr", parv[1]);
		return;
	}
	strncpy(p, parv[4], 15);
	if(strlen(p) < 6)
	{
		uwntc(nick->numeric, "Le pass doit faire entre 6 et 15 caractres.");
		return;
	}
	if(strlen(parv[2]) > 140)
	{
		uwntc(nick->numeric, "Le mail est limit  140 charactres.");
		return;
	}
	if(!strchr(parv[2], '@') || !strchr(parv[2], '.'))
	{
		uwntc(nick->numeric, "Veuillez entrer un email correct.");
		return;
	}
	strcpy(passwd, mkpass(parv[4], "lala"));

	add_regnick(parv[1], passwd, time(NULL), niveau, 0, 0, parv[2]);
	uwntc(nick->numeric, "L'user %s a bien t ajout comme admin niveau %d. Il peut s'identifier avec LOGIN", parv[1], niveau);
	write_userlist();
	write_anuser(parv[1]);
	return;
}

/*
 * admin_level
 *
 * parv[1] = username
 * parv[2] = niveau
 */
void admin_level(aNick *nick, int parc, char **parv)
{
	anUser *user;
	int niveau, last;

	if(parc<2)
	{
		syntax_cmd(uw.nick,nick->numeric,"ADMINLVL");
		return;
	}
	if(!is_num(parv[2]))
	{
		uwntc(nick->numeric, "Le niveau doit tre un nombre.");
		return;
	}
	niveau = atoi(parv[2]);
	if(niveau > 4 || niveau < 1)
	{
		uwntc(nick->numeric, "Le niveau des admins doit tre entre 1 et 4.");
		return;
	}
	if(!(user = getuserinfo(parv[1])))
	{
		uwntc(nick->numeric, NOSUCHUSER, parv[1]);
		return;
	}
	if(!IsAnAdmin(user))
	{
		uwntc(nick->numeric, "%s n'est pas Admin.", user->nick);
		return;
	}
	if(nick->user && niveau >= nick->user->level && nick->user->level != 4)
	{
		uwntc(nick->numeric, "Vous ne pouvez pas mettre %s  un niveau suprieur ou gal au votre.", user->nick);
		return;
	}
	last = user->level;
	if(last == niveau)
	{
		uwntc(nick->numeric, "%s est dj au niveau %d !", user->nick, niveau);
		return;
	}
	user->level = niveau;
	uwntc(nick->numeric, "%s a maintenant le niveau %d", user->nick, user->level);
	write_anuser(user->nick);
	return;
}

/*
 * del_admin
 *
 * parv[1] = pseudo
 */
void del_admin(aNick *nick, int parc, char **parv)
{
	anUser *user;

	if(!parc)
	{
		syntax_cmd(uw.nick,nick->numeric,"DELADMIN");
		return;
	}
	if(!(user = getuserinfo(parv[1])))
	{
		uwntc(nick->numeric, NOSUCHUSER, parv[1]);
		return;
	}
	if(!IsAnAdmin(user))
	{
		uwntc(nick->numeric, "%s n'est pas Admin", user->nick);
		return;
	}
	del_regnick(parv[1]);
	uwntc(nick->numeric, "%s n'est plus Admin", user->nick);
}

void uw_whois(aNick *nick, int parc, char **parv)
{
	char *user, chans[1000], achans[1000], b[205];
	int r = 0;
	aNick *tmp;
	aJoin *chanl;
	anUser *ux;

	if(parc<1)
	{
		uwntc(nick->numeric, "Syntaxe: WHOIS <nick>");
		return;
	}
	if(strlen(parv[1]) > 40) return;

	user = parv[1];
	if(!(tmp = getnickbynick(user)))
	{
		uwntc(nick->numeric, NOTCONNECTED, user);
		return;
	}

	chans[0] = '\0';
	achans[0] = '\0';
	chanl = tmp->joinhead;
	while(chanl)
	{
		snprintf(b, 250, "%s%s%s ", chanl->is_voice ? "+" : "", chanl->is_op ? "@" : "", chanl->channel);
		strncat(chans, b, 1000);
		chanl = chanl->next;
	}

	if(tmp->user)
		r = 1;

	if(chans[0] == '\0') sprintf(chans, "Aucun Salon");

	uwntc(nick->numeric, "Infos sur \002%s", tmp->nick);
	uwntc(nick->numeric, " Nom: %s", tmp->name);
#ifdef HAVE_CRYPTHOST
	uwntc(nick->numeric, " Mask: \002%s\002!\002%s\002@\002%s", tmp->nick, tmp->ident, tmp->crypt);
	uwntc(nick->numeric, " Realhost: \002%s\002", tmp->host);
#else
	uwntc(nick->numeric, " Mask: \002%s\002!\002%s\002@\002%s", tmp->nick, tmp->ident, IsAnAdmin(nick->user) ? tmp->host : "*");
#endif
	//uwntc(nick->numeric, "IP: \002%s", dcdip(tmp->base64));
	if(tmp->is_oper) uwntc(nick->numeric, " %s est \002IRCop.", tmp->nick);
	uwntc(nick->numeric, " Il est sur: %s", chans);
	if(r) uwntc(nick->numeric, " %s est logu sous l'username \002%s\002", user, tmp->user->nick);
	else if((ux = getuserinfo(user))) uwntc(nick->numeric, " %s est un username enregistr. %s", ux->level);
	uwntc(nick->numeric, " Connect depuis %s sur \002%s", duration(time(NULL) - tmp->ttmco), tmp->server);
	return;
}

void admin_match(aNick *nick, int parc, char **parv)
{
   aNick *n;
   char host[USERLEN + HOSTLEN], *req, b[100], *x = NULL;
   int i = 0, d = 0;
   struct timeval tv;
   long ticks;
   gettimeofday(&tv, NULL);
   ticks=tv.tv_usec;

   if(!parc)
   {
	   uwntc(nick->numeric, "Syntaxe ADMINMATCH <mask> [nombre]");
	   return;
   }

   req = parv[1];
   if(parc>1) x = parv[2];
   if(x != NULL) d = atoi(x);
   uwntc(nick->numeric, "Listing Nicks...");

   n = nickhead;
   while(n)
   {
	   snprintf(host, (USERLEN + HOSTLEN), "%s@%s", n->ident, n->host);
	   if(!match(req, host))
	   {
		   if(!d || (i < d)) uwntc(nick->numeric, "Matching Nick: \002%s%s\002 -> %s", n->is_oper ? "\0037*\003" : "", n->nick, host);
		   i++;
	   }
	   n = n->next;
   }
   if(d && (i > d)) sprintf(b, " (%d entres listes.)", d);
   else sprintf(b, " ");

   gettimeofday(&tv, NULL);
   uwntc(nick->numeric, "Fin de la Recherche sur\002 %s\002 : %d matche%s trouv%s%s. (Effectue en %6.3f ms)"\
   , req, i, (i > 1) ? "s" : "", (i > 1) ? "s" : "", b,(float) (tv.tv_usec - ticks)/1000);
   return;

}

char *getuserbyhost(char *host)
{
	aNick *nick;
	char *n;
	n = malloc(5000);
	n[0]='\0';
	nick=nickhead;
	while(nick)
	{
		if(!strcasecmp(nick->host,host))
		{
			if(n[0]!='\0') strcat(n, " ");
			strncat(n, nick->nick, 5000);
		}
		nick=nick->next;
	}
	return n;
}

int istk(char **p, char *h)
{
	int i=0;
	while(p[i])
	{
		if(!strcasecmp(p[i],h)) return 1;
		i++;
	}
	return 0;
}

void admin_user(aNick *nick, int parc, char **parv)
{
	char *arg, *user, b[300];
	int m = 0, x = 0;
	anUser *u;
	aNick *n;

	arg = parv[1];
	user = parv[2];
	if(!strcasecmp(arg, "info"))
	{
		if(strlen(user)>40) return;
		if(strchr(user, '*') || strchr(user, '?')) m = 1;
		u = userhead;
		while(u)
		{
			if((!m && !strcasecmp(u->nick, user)) || (m && !match(user, u->nick)))
			{
				uwntc(nick->numeric, "Infos sur l'username \002%s\002 :", u->nick);
				uwntc(nick->numeric, " Mail: %s", u->mail);
				uwntc(nick->numeric, " Logu pour la dernire fois il y a %s", duration(time(NULL) - u->lastseend));
				uwntc(nick->numeric, " \002Accs Administrateur.\002 (\002%d\002)", u->level);
				b[0] = '\0';
				if((n = getnickinfo(u->nick))) uwntc(nick->numeric, "%s est actuellement logu sous cet Username (%s@%s)",\
				n->nick, n->ident, n->host);
				else uwntc(nick->numeric, "Personne n'est actuellement logu sous cet username");
				uwntc(nick->numeric, " Il %s suspendu, le SetKill est %s", u->suspendu ? "est" : "n'est pas",\
				u->setkill ? "actif" : "inactif");
				x++;
				if(m) uwntc(nick->numeric, "-");
				else return;
			}
			u = u->next;
		}
		if(m && x) uwntc(nick->numeric, "Un Total de %d entre%s trouve%s", x, x ? "s" : "", x ? "s": "");
		if(!x) uwntc(nick->numeric, "Aucun Username correspondand  \002%s\002 trouv.", user);
		return;
	}
	if(!strcasecmp(arg, "suspend"))
	{
		if(!strcasecmp(nick->user->nick, user))
		{
			uwntc(nick->numeric, "Vous ne devriez pas vous suspendre.. (Utilisez la commande SUSPENDME)");
			return;
		}
		u = userhead;
		while(u)
		{
			if(!strcasecmp(u->nick, user))
			{
				if(u->level > 1 && u->level >= nick->user->level)
				{
					uwntc(nick->numeric, "Vous ne pouvez pas suspendre un Admin d'un niveau suprieur ou quivalent au votre.");
					return;
				}
				u->suspendu = u->suspendu ? 0 : 1;
				uwntc(nick->numeric, "L'username \002%s\002 est bien %ssuspendu.", u->nick, u->suspendu ? "" : "de");
				if((n = getnickinfo(u->nick)))
				{
					n->user->authed = u->suspendu ? 0 : 1;
					u->authed = u->suspendu ? 0 : 1;
					uwntc(n->numeric, "Votre Username vient d'tre %ssuspendu par l'Admin %s.", u->suspendu ? "" : "de", nick->nick);
				}
				write_anuser(u->nick);
				return;
			}
			u = u->next;
		}
		uwntc(nick->numeric, NOSUCHUSER, user);
		return;
	}
	if(!strcasecmp(arg, "mail"))
	{
		if(parc < 3 || !strchr(parv[3], '@') || !strchr(parv[3], '.') || strlen(parv[3]) > 148)
		{
			uwntc(nick->numeric, "Veuillez prciser un Mail valide.");
			return;
		}
		if(!(u = getuserinfo(user)))
		{
			uwntc(nick->numeric, NOSUCHUSER, user);
			return;
		}
		strncpy(u->mail, parv[3], 150);
		uwntc(nick->numeric, "Le mail de %s a t chang en \002%s", u->nick, parv[3]);
		write_anuser(user);
		return;
	}
	if(!strcasecmp(arg, "newpass"))
	{
		char p[15];
		if(parc < 3 || strlen(parv[3]) < 6)
		{
			uwntc(nick->numeric, "Veuillez entrer un pass (Valide).");
			return;
		}
		if(!(u = getuserinfo(user)))
		{
			uwntc(nick->numeric, NOSUCHUSER, user);
			return;
		}
		strncpy(p,parv[3],15);
		strcpy(u->passwd, mkpass(p, "lala"));
		uwntc(nick->numeric, "Le mot de pass de %s a t chang.", user);
		write_anuser(user);
		return;
	}
	if(!strcasecmp(arg, "del"))
	{
		char fichier[230], *rnick;
		if(!strcasecmp(nick->user->nick, user))
		{
			uwntc(nick->numeric, "Vous ne devriez pas vous deluser.. (Utilisez la commande DROP)");
			return;
		}
		if(!(u = getuserinfo(user)))
		{
			uwntc(nick->numeric, NOSUCHUSER, user);
			return;
		}
		if(u->level > 1 && u->level >= nick->user->level)
		{
			uwntc(nick->numeric, "Vous ne pouvez supprimer un Username d'Admin de niveau suprieur ou gal.");
			return;
		}
		if((n = getnickinfo(user)))
		{
			putserv(":%s SVSMODE %s -r\n", uw.nick, n->numeric);
			uwntc(n->numeric, "Votre Username vient d'tre supprim par l'Admin %s %s", nick->nick, (parc > 2) ? parv2msg(parc, parv, 3) : "");
			n->user->authed = 0;
			n->user = NULL;
		}

		uwntc(nick->numeric, "L'username %s a bien t supprim.", u->nick);
		rnick = conv_char(u->nick);
		del_regnick(u->nick);
		snprintf(fichier, 100, "database/usr/%s", rnick);
		remove(fichier);
		write_userlist();
		return;
	}
	if(!strcasecmp(arg, "stats"))
	{
		if(!strcasecmp(user, "cmds"))
		{
			if(parc > 2) m = 1;
			for(x=0;uw_cmds[x].cmd[0]!='\0';x++)
			{
				if((!m || !match(parv[3],uw_cmds[x].cmd)) && uw_cmds[x].cmd[0]!='\001') uwntc(nick->numeric, "- %s (\002%d\002)", uw_cmds[x].cmd, uw_cmds[x].used);
			}
			return;
		}
		if(!strcasecmp(user, "traffic"))
		{
			float up,down;
			int mbu=0,mbd=0;
			up=dataS/1024.0;
			down=dataQ/1024.0;
			if(up>=1024)
			{
				up=up/1024.0;
				mbu=1;
			}
			if(down>=1024)
			{
				down=down/1024.0;
				mbd=1;
			}
			uwntc(nick->numeric, "Traffic: Up: \2%10.3f %s\2 Down: \2%10.3f %s\2", up, mbu ? "MB":"KB", down, mbd ? "MB":"KB");
			return;
		}
  		if(!strcasecmp(user, "users"))
		{
			struct timeval tv;
			int tuser = 0,th=0;
			char *host[500], *l, *o = NULL;
			long ticks;
			gettimeofday(&tv, NULL);
			ticks=tv.tv_usec;
			host[0] = NULL;
			n = nickhead;
			while(n)
			{
				tuser++;
				if(!istk(host,n->host))
				{
					if(strchr((l=getuserbyhost(n->host)), ' '))
					{
						if(o!=l && o) free(o);
						o=l;
						uwntc(nick->numeric, "clones sur \002%s\002 : %s", n->host, l);
						host[th++] = n->host;
					}
				}
				n=n->next;
			}
			if(o) free(o);
			gettimeofday(&tv, NULL);
			uwntc(nick->numeric, "Il y a \002%d\002 Utilisateurs sur les serveurs. (\002%d\002 hosts prsentent des clones) Effectu en : \002%6.3f\002 ms",tuser,th,(float) (tv.tv_usec - ticks)/1000);

			return;
		}
		uwntc(nick->numeric, "Option de stats non valide");
		return;
	}
	/*if(!strcasecmp(arg, "test"))
	{
		char *p;
		int pi=0,pp=0;
		if(parc < 2) return;
		p=parv[2];
		while(!pp)
		{
			if((p=strchr(p,'.')))
			{
				uwntc(nick->numeric, "test: %s", p);
				p++;
				pi++;
			}
			else pp = 1;
		}
		uwntc(nick->numeric, "nb . %d", pi);
		return;
	}*/
	uwntc(nick->numeric, "%s n'est pas une option valide.", arg);
	return;
}

void whoison(aNick *nick, int parc, char **parv)
{
	char *c;
	c = parv[1];
	uwntc(nick->numeric, "Users sur \002%s", c);
	uwntc(nick->numeric, getchanuser(c));
	return;
}

void admin_say(aNick *nick, int parc, char **parv)
{
	char *tmp = NULL;

	tmp = parv2msg(parc, parv, 2);
	if(*tmp == ':') tmp++;
	putserv(":%s PRIVMSG %s :%s\n", uw.nick, parv[1], tmp);
	return;
}

void admin_do(aNick *nick, int parc, char **parv)
{
	char *tmp = NULL;

	tmp = parv2msg(parc, parv, 2);
	if(*tmp == ':') tmp++;
	putserv(":%s PRIVMSG %s :\001ACTION %s\001\n", uw.nick, parv[1], tmp);
	return;
}

void die(aNick *nick, int parc, char **parv)
{
	char r[200];
	write_users();
	write_userlist();
	write_badchans();
	want_to_quit = 1;
	if(!parc) strcpy(r, quit_msg);
	else strncpy(r, parv2msg(parc, parv, 1), 199);
	putserv(":%s QUIT :%s (par \002%s\002)\n", uw.nick, r, nick->nick);
	putserv(":%s SQUIT %s 0 :%s (par \002%s\002)\n", bot.server, bot.server, r, nick->user->nick);
	return;
}

/*
 * restart_bot
 *
 * parv[1->parc-1] = reason
 */
void restart_bot(aNick *nick, int parc, char **parv)
{
	FILE *fuptime;

	if(!parc) parv[++parc]=quit_msg;
	parv[++parc]="[\002RESTART\002]";
	if((fuptime = fopen("uptime.tmp", "w"))) // permet de maintenir l'uptime (/me tricheur)
	{
		fprintf(fuptime, "%d %d %d", tuptime, lastresetparse, lastresetlogs);
		fclose(fuptime);
		/* Comme je m'emmerde, voici un c/c de ce qu'il se passe grce  a :
		 -*-> (183552) @Progs > !uptime
		 -*-> (183552) -Z- Je suis connect depuis 20 heures 31 minutes et 14 secondes
		 -*-> (183555) @Progs > !restart
		 -*-> (183556) <== (QUIT) Z (SCoderZ@Service.Inter-System.Net) Raison: Maintenance [RESTART] (par Progs)
		 -*-> (183623) -Z- Vous avez t ridentifi
		 -*-> (183623) -Z/Wallops- Synchronisation de 581 bytes en     0.002405 s (Taux   235.918017 Ko/s)
		 -*-> (183623) ==> (JOIN) Z (SCoderZ@Service.Inter-System.Net) sur #Services
		 -*-> (183623) === (MODE) #Services +o Z par Services.Inter-System.Net
		 -*-> (183626) @Progs > !uptime
		 -*-> (183626) -Z- Je suis connect depuis 20 heures 31 minutes et 48 secondes
		 * comme (18:36:26) - (28:35:52) = (20:31:48) - (20:31:14) a marche :)
		 * dsl pour le flood
		 */
	}
	die(nick, parc, parv);
	system("./services");
	return;
}

void uw_wall(aNick *nick, int parc, char **parv)
{
	aNick *n;
	char *buf = parv2msg(parc,parv,1);
	if(*buf == ':') buf++;
	for(n=nickhead;n;n=n->next) if(n->user && IsAnAdmin(n->user)) uwntc(n->numeric, "UWWall de \002%s\002: %s", nick->nick, buf);
	return;
}

void chcomname(aNick *nick, int parc, char **parv)
{
	char newcmd[15], *lastcmd = NULL, chhelp[100]/*, cmd1[50], cmd2[50]*/;
	int i, icmd = 0;

	chhelp[0]='\0';
	if(parc < 2)
	{
		syntax_cmd(uw.nick, nick->numeric, "chcomname");
		return;
	}
	if(strlen(parv[2]) > 15)
	{
		uwntc(nick->numeric, "Un nom de commande ne doit pas dpasser 15 caractres");
		return;
	}
	strcpy(newcmd, parv[2]);
	lastcmd = parv[1];
	for(i=0;uw_cmds[i].cmd[0] != '\0';i++)
		if(!strcasecmp(lastcmd, uw_cmds[i].cmd))
		{
			if(uw_cmds[i].cmd[0] == '\001')
			{
				uwntc(nick->numeric, "Vous ne pouvez pas changer le nom des commandes en ctcp");
				return;
			}
			icmd = i;
		}
		else if(!strcasecmp(newcmd, uw_cmds[i].cmd))
		{
			uwntc(nick->numeric, "La commande %s existe dj", newcmd);
			return;
		}
	if(icmd)
	{
		strcpy(uw_cmds[i].realcmd, uw_cmds[i].cmd);
		strcpy(uw_cmds[i].cmd, newcmd);

		/*cmd1[0] = '\0';
		sprintf(cmd1, "%s", strupr(lastcmd));
		cmd2[0] = '\0';
		sprintf(cmd2, "%s", strupr(newcmd));*/
		strupr(lastcmd);
		strupr(newcmd);
		//printf("%s %s %s\n", chhelp, cmd1, cmd2);

		system(chhelp);

		uwntc(nick->numeric, "La commande \002%s\002 est appele maintenant \002%s\002", lastcmd, uw_cmds[i].cmd);
	}
	else
		uwntc(nick->numeric, "La commande \002%s\002 existe pas !", lastcmd);
	return;
}

void chlevel(aNick *nick, int parc, char **parv)
{
	char *cmd = NULL;
	int level = 0, i;

	if(parc < 2)
	{
		syntax_cmd(uw.nick, nick->numeric, "chlevel");
		return;
	}
	cmd = parv[1];
	if(!is_num(parv[2]) || atoi(parv[2]) > 500 || atoi(parv[2]) < 0)
	{
		uwntc(nick->numeric, "Un niveau doit tre un nombre entre 0 et 500");
		return;
	}
	level = atoi(parv[2]);
	if(nick->user && level > nick->user->level)
	{
		uwntc(nick->numeric, "\002Vous ne pouvez pas donner un niveau  %s plus grand que le votre !\002", cmd);
		return;
	}
	for(i=0;uw_cmds[i].cmd[0] != '\0';i++)
		if(!strcasecmp(cmd, uw_cmds[i].cmd))
		{
			if(level > 4)
			{
				uwntc(nick->numeric, "Le niveau d'une commande d'user ne doit pas dpasser le niveau 4");
				return;
			}
			uw_cmds[i].level = level;
			uwntc(nick->numeric, "Le niveau de la commande %s est maintenant \002%d\002", uw_cmds[i].cmd,uw_cmds[i].level);
			return;
		}
	uwntc(nick->numeric, "La commande \002%s\002 existe pas !", cmd);
	return;
}

void disable_cmd(aNick *nick, int parc, char **parv)
{
	char *cmd = NULL;
	int disable = 0, i;

	if(parc < 2)
	{
		syntax_cmd(uw.nick, nick->numeric, "disable");
		return;
	}
	cmd = parv[1];
	if(!strcasecmp(parv[2], "ON")) disable = 1;
	else if(!strcasecmp(parv[2], "OFF")) disable = 0;
	else
	{
		uwntc(nick->numeric, "Vous devez prciser la commande est desactive grace  ON ou OFF");
		return;
	}
	for(i=0;uw_cmds[i].cmd[0] != '\0';i++)
		if(!strcasecmp(cmd, uw_cmds[i].cmd))
		{
			if(!strcasecmp(uw_cmds[i].realcmd, "DISABLE") || !strcasecmp(uw_cmds[i].realcmd, "CHCOMNAME") || !strcasecmp(uw_cmds[i].realcmd, "CHLEVEL"))
			{
				uwntc(nick->numeric, "Veuillez ne pas desactiver %s !", cmd);
				return;
			}
			uw_cmds[i].disable = disable;
			uwntc(nick->numeric, "La commande %s est maintenant \002%sactive\002", uw_cmds[i].cmd, disable ? "des" : "");
			return;
		}
	uwntc(nick->numeric, "La commande \002%s\002 existe pas !", cmd);
	return;
}

/*
 * add_badchan
 *
 * parv[1] = chaine
 * parv[2-parc] = raison
 */
void add_badchan(aNick *nick, int parc, char **parv)
{
	char chaine[100], raison[150];
	struct badchans *badchan;

	if((badchan = getbadchan(parv[1])))
	{
		uwntc(nick->numeric, "%s est dj couvert par \002%s\002 (\002%s\002)\002.", badchan->chaine, badchan->raison);
		return;
	}
	strncpy(chaine, parv[1], 100);
	strncpy(raison, parv2msg(parc, parv, 2), 150);/*pas de boucles inutiles avt les check*/
	add_abadchan(chaine, raison);
	uwntc(nick->numeric, "La chaine \002%s\002 a bien t ajoute avec la raison: \002%s", chaine, raison);
	uwntc(nick->numeric, "Aucun salon contenant cette chaine ne pourra tre enregistr par les Users.");
	write_badchans();
	return;
}

/*
 * del_badchan
 *
 * parv[1] = chaine
 */
void del_badchan(aNick *nick, int parc, char **parv)
{
	char *chaine = parv[1];
	struct badchans *badchan;

	if(!(badchan = getbadchan(chaine)))
	{
		uwntc(nick->numeric, "La chaine %s n'existe pas", chaine);
		return;
	}
	del_abadchan(badchan->chaine);
	uwntc(nick->numeric, "La chaine \002%s\002 a bien t supprime", chaine);
	write_badchans();
	return;
}

void list_badchans(aNick *nick, int parc, char **parv)
{
	struct badchans *current;

	uwntc(nick->numeric, "\002\037Liste des salons illgaux :");
	for(current=badchead;current;current=current->next) uwntc(nick->numeric, "  \002%s\002 : %s", current->chaine, current->raison);
	uwntc(nick->numeric, "Les salons illgaux sont vrifis avec des * et des ? comme les bans.");
    return;
}

// kill <mask> [raison]
void kill_user(aNick *nick, int parc, char **parv)
{
	aNick *vi;
	char *raison = "Gros Vilain !";
	int wildcards = 0, i=0, opers=0;

	if(!parc)
	{
		uwntc(nick->numeric, "Syntax: KILL <pseudo> <raison>");
		return;
	}
	wildcards = (strchr(parv[1], '*') || strchr(parv[1], '?'));
	for(vi=nickhead;vi;vi=vi->next)
		if((wildcards && !match(parv[1], vi->nick)) || (!wildcards && !strcasecmp(parv[1], vi->nick)))
		{
			if(vi->is_oper) opers++;
			else
			{
				putserv(":%s KILL %s :%s\n", uw.nick, vi->numeric, raison);
				del_nickinfo(vi->numeric);
			}
			i++;
			if(!wildcards) break;
		}
	if(wildcards)
	{
		if((i - opers)>1) uwntc(nick->numeric, "[\2%d\2] users ont t vrifis sur \2%s\2", (i - opers), parv[1]);
		else if((i - opers)) uwntc(nick->numeric, "[\2%d\2] user a t vrifi sur \2%s\2", (i - opers), parv[1]);
		else uwntc(nick->numeric, "[\2Aucun\2] user n'a t vrifi sur \2%s\2", parv[1]);
		if(opers) uwntc(nick->numeric, "Plus %d IrcOp qui n'ont PAS t kill", opers);
	}
	else if((i - opers)) uwntc(nick->numeric, "\2%s\2 s'est bien fait kill", parv[1]);
	else if(opers) uwntc(nick->numeric, "\2%s\2 ne peut pas tre kill: c'est un irc oprateur", parv[1]);
	else uwntc(nick->numeric, NOTCONNECTED, parv[1]);
	return;
}
