/* src/main.c - Fichier principal
 * Copyright (C) 2002 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <unistd.h>
#include <signal.h>
#include <time.h>
#define GLOBALS_DEF
#include "main.h"
#undef GLOBALS_DEF

CVSINFO("$Id: main.c,v 1.1.1.1 2002/11/03 20:16:59 coderz Exp $");

/*
 *            FONCTIONS COMMANDES
 *            =-=-=-=-=-=-=-=-=-=
 *
 *    Il y a deux types de fonctions commandes.
 *    Les normales :
 *       parv[0] = pseudo du robot appel
 *       parv[1->parc] = arguments de la commandes
 *    Et les commandes salons (voir struct uw_commands) :
 *       parv[0] = pseudo du robot appel
 *       parv[1] = salon appel
 *       parv[2->parv] = arguments de la commande (aprs le nom du salon)
 *    Toute les commandes doivent respecter ce shema.
 *    Lorsqu'une commande salon est tap sur un salon, et que le premier
 *    argument n'est pas un nom de salon, le nom du salon actuel est plac
 *    automatiquement comme premier argument.
 *
 *    Pour faciliter la lecture, et au cas o il pouvait y avoir plusieurs cas
 *    (ex: cmd op, if (parc == 0) cible = nick->nick else cible = parv[1]).
 *
 *    Chaque commande emploi un entte particuli :
 *    void commande(aNick *nick, int parc, char **parv)
 *      aNick *nick = structure nickinfo de l'user ki a appel la fonction
 *      int parc = nombre d'arguments *SANS COMPTER PARV[0]*. (si le dernier arg
 *                 est parv[5], alors parc = 5.)
 *      char **parv = tableau contenant les arguments (une ligne = un mot).
 *
 *    Outils utiles :
 *      priv2msg(int parc, char **parv, int position)
 *          permet de convertire une fin de tableau en un message normal.
 *          il faut spcifier le tableau et le nombre d'arguments contenu
 *          ainsi que la position  laquelle on va commencer  enregistrer
 *          le contenu dans une variable qui sera renvoye.
 *      nick2num(char nick), num2nick(char num)
 *          renvoi la num d'un pseudo spcifi dans l'un, et le pseudo d'une
 *          num spcifie dans l'autre.
 *    Pour plus d'outils, voir outils.c.
 */

void sig_handler (int crap)
{
  want_to_quit = 1;
  signal(SIGINT, &sig_handler);
  signal(SIGTERM,&sig_handler);
}

void sig_reload (int crap)
{
  signal(SIGHUP, &sig_reload);
}

int lastcheckaccounts = 0;

void sig_alarm(int crap)
{
	check_kills();
	check_ignores();
	if((lastresetparse + WAIT_DEL_PARSES) < time(NULL))
	{
		remove(LOG_PARSES);
		lastresetparse = time(NULL);
	}
	if((lastresetlogs + WAIT_DEL_LOGS) < time(NULL))
	{
		remove(LOG_CMDS);
		lastresetlogs = time(NULL);
	}
	alarm(ALARMINTERVAL);
}

int main(int argc, char **argv)
{
  int sock;
  int irc_ip;
  int r, i, restarted = 0;
  aNick *nick;
  FILE *fuptime;
  char buff[100];

  if (load_config(FICHIER_CONF)== (-1))
  {
    printf("Erreur lors de la lecture de la configuration\n");
    exit(-1);
  }

  if((fuptime = fopen("uptime.tmp", "r")))
  {
	while(fgets(buff, 420, fuptime) != NULL)
	{
		strip_newline(buff);
		sscanf(buff, "%d %d %d",&tuptime, &lastresetparse, &lastresetlogs);
	}
	fclose(fuptime);
	remove("uptime.tmp");
	restarted = 1;
  }

  load_cmds();
  load_users();
  load_badchans();

  STS = time(NULL);
  signal(SIGPIPE, SIG_IGN);
  signal(SIGHUP, &sig_reload);
  signal(SIGINT, &sig_handler);
  signal(SIGTERM,&sig_handler);
  signal(SIGALRM, &sig_alarm);

  r=fork();

  if (r == -1)
  {
    printf("Impossible d'tablir une connexion.");
    exit(-1);
  }

	if (r > 0)
	{
		if(!restarted)
		printf("UWorlD CoderZ %s  Inter System\n", SPVERSION);
		if(premiere)
		{
			printf("Lorsque les services seront sur votre rseau IRC,\n");
			printf("tapez : /%s hello\n", uw.nick);
		}
		exit(0);
	}

	strncpy(bot.servnum, make_servnum(bot.num), 16);

	strncpy(uw.num, make_servnum(bot.num), 13);
	strcat(uw.num, "UW");

	if(!premiere)
	{
		for (i=0; uw_cmds[i].cmd[0] != '\0'; i++)
		{
			if (((char *)strcasecmp(uw_cmds[i].cmd,"hello"))==NULL)
			{
				uw_cmds[i].disable = 1;
				break;
			}
		}
	}

  while (want_to_quit==0)
  {
    irc_ip = resolve(bot.ip);
    if ((irc_ip == 0) || (irc_ip == -1))
    {
      sock = -1;
    } else sock = init_bot(irc_ip, bot.port);
    bot.ipnum = irc_ip;

    if (sock<0)
    {
      putlog(LOG_PARSES, "Connection choue\n");
    } else
    {
      run_bot(sock);
    }
    if (want_to_quit==1)
    {
      close(sock);
    } else
    {
      putlog(LOG_PARSES, "Dconnect par le serveur\n");
      for(nick = nickhead;nick;nick = nickhead)
      {
        if(nick->user != NULL) nick->user->authed = 0;
	del_nickinfo(nick->numeric);
      }
      close(sock);
      sleep(WAIT_CONNECT);
    }
  }
  putlog(LOG_PARSES, "Fermeture normale du programme\n");
  return (0);
}
