/* src/outils.c - Divers outils
 * Copyright (C) 2002 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <time.h>
#include <stdarg.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <main.h>
#ifdef HAVE_CRYPTHOST
#include <crypt.h>
#endif

CVSINFO("$Id: outils.c,v 1.1.1.1 2002/11/03 20:16:59 coderz Exp $");

static char *months[] = {
  "Janvier", "Fevrier", "Mars", "Avril",
  "Mai", "Juin", "Juillet", "Aot",
  "Septembre", "Octobre", "Novembre", "Decembre"
};

static char *weekdays[] = {
  "Dimanche", "Lundi", "Mardi", "Mercredi",
  "Jeudi", "Vendredi", "Samedi"
};

#ifdef HAVE_CRYPTHOST
char *hostprot(unsigned char *key, unsigned char *host)
{
  u_int32_t sum;
  u_int32_t digest[4];
  char *newhost;
  char ID[64];
  char *ptr;

  if(!host[0]%2)
  { // md2
     MD2_CTX context;
     MD2Init(&context);
     MD2Update(&context,key,strlen(key));
     MD2Update(&context,host,strlen(host));
     MD2Final((void *) digest,&context);
  }
  else
  { // md5
     MD5_CTX context;
     MD5Init(&context);
     MD5Update(&context,key,strlen(key));
     MD5Update(&context,host,strlen(host));
     MD5Final((void *) digest,&context);
  }

  sum=digest[0]+digest[1]+digest[2]+digest[3];

  sprintf(ID,"%X", sum);
  newhost = malloc(500);
  newhost[0] = '\0';
  if (!is_ip(host))
  {
    ptr=(char *) strchr(host, '.');
    strncpy(newhost, ID,256);
    strncat(newhost, ptr, 245);
  } else
  {
    ptr=(char *) strrchr(host, '.');
    strncpy(newhost,host,((int)ptr-(int)host) +1 );
    newhost[((int)ptr-(int)host)+1]=0;
    newhost[245]=0;
    strncat(newhost, ID, 10);
  }
  return(newhost);
}
#endif

int is_num(char *num)
{
   while(*num)
   {
     if(!isdigit(*num)) return 0;
     num++;
   }
   return 1;
}

int is_ip(char *addr)
{
   while(*addr)
   {
     if(!isdigit(*addr)&&( *addr != '.' )) return 0;
     addr++;
   }
   return 1;
}

int countdots(char *addr)
{
	int pi=0;
	while((addr=strchr(addr,'.')))
	{
		addr++;
		pi++;
	}
	return pi;
}

void strupr(char *buf)
{
   int i = 0;
   while(buf[i] != '\0')
   {
      buf[i] = toupper(buf[i]);
      i++;
   }
   buf[i] = '\0';
}

void strlwr(char *buf)
{
   int i = 0;
   while(buf[i] != '\0')
   {
      buf[i] = tolower(buf[i]);
      i++;
   }
   buf[i] = '\0';
}

char *conv_char(char *chan)
{
	char val[50];
	int i, chanlen = strlen(chan);

	val[0]='\0';

	tbuff[0]='\0';
	for(i = 0; i < chanlen; i++)
	{
		if(!isalnum(chan[i])) sprintf(val, "%%%03o", chan[i]);
		else sprintf(val, "%c", chan[i]);
		strncat(tbuff, val, 230);
	}

	return tbuff;
}

void strip_newline(char *string)
{
   char *ptr = NULL;
   if((ptr = strchr(string, '\n'))) *ptr = '\0';
   if((ptr = strchr(string, '\r'))) *ptr = '\0';
}

struct joininfo *getjoininfo(char *nick, char *chan)
{
   aNick *current;
   aJoin *join;

   current = nickhead;
   while(current)
   {
      if(!strcasecmp(nick,current->nick))
      {
         for(join = current->joinhead;join && strcasecmp(chan, join->channel);join=join->next);
		 return join;
      }
      current = current->next;
   }
   return NULL;
}

struct userinfo *getuserinfo(char *nick)
{
   struct userinfo *current;
   for(current = userhead;current && strcasecmp(nick,current->nick);current=current->next);
   return current;
}

struct nickinfo *getnickinfo(char *nick)
{
   struct nickinfo *current;
   current = nickhead;
   while(current)
   {
      if(current->user && !strcasecmp(nick, current->user->nick)) return current;
      current = current->next;
   }
   return NULL;
}

struct nickinfo *getnickbynick(char *nick)
{
   struct nickinfo *current;
   current = nickhead;
   while(current)
   {
      if(!strcasecmp(nick,current->nick)) return current;
      current = current->next;
   }
   return NULL;
}

struct badchans *getbadchan(char *chan)
{
   struct badchans *current;
   current = badchead;
   while(current && match(current->chaine, chan)) current = current->next;
   return current;
}

char *getchanuser(char *chan)
{
	aNick *nick;
	aJoin *chans;
	char *list, c[200];
	list=malloc(1000);
	c[0] = '\0';
	list[0] = '\0';
	nick = nickhead;
	while(nick)
	{
		chans = nick->joinhead;
		while(chans)
		{
			if(!strcasecmp(chans->channel, chan))
			{
				snprintf(c, 200, " %s", nick->nick);
				strncat(list, c,1000);
				break;
			}
			chans = chans->next;
		}
		nick = nick->next;
	}
	return list;
}

char *num2nick(char *num)
{
   struct nickinfo *current;

   for(current = nickhead;current;current=current->next) if(!strcmp(num, current->numeric)) return current->nick;
   return NULL;
}

char *nick2num(char *nick)
{
   struct nickinfo *current;

   for(current = nickhead;current;current=current->next) if(!strcasecmp(nick, current->nick)) return current->numeric;
   return NULL;
}

char *num2serv(char num)
{
   int i = 0;
   char *entry=(char *)NULL;

   for(i=0;i<NUMNICKBASE;i++)
   {
      if(num == servtab[i].chars)
      {
         entry=servtab[i].name;
         break;
      }
   }
   return(entry);
}

int b2n (int b)
{
  int i;
  for (i=0; i<64; i++)
  {
    if (base64[i] == b) return i;
  }
  return -1;
}

int decode_num (char *s)
{
  int i,c,poids,num;
  num=0;
  poids=1;
  for (i=strlen(s)-1; i >= 0; i--)
  {
    c = b2n(s[i]);
    if (c == -1) return -1;
    num += poids*c;
    poids=poids*64;
  }
  return num;
}

int findserv (char *name)
{
  int i;
  for (i=0; i<64; i++)
  {
    if (!strcasecmp(name, servtab[i].name)) return i;
  }
  return -1;
}

void log(char *filename, char *format, ...)
{
   va_list arglist;
   FILE *fp;
   if(!(fp = fopen(filename, "a"))) return;
   va_start(arglist, format);
   vfprintf(fp, format, arglist);
   va_end(arglist);
   fclose(fp);
}

char *parv2msg(int parc, char **parv, int base)
{
  int i;
  parvbuff[0] = '\0';

  for(i=base;i<=parc;i++)
  {
    if(i>base) strcat(parvbuff, " ");
    strncat(parvbuff, parv[i], 500);
  }
  return parvbuff;
}

char *duration(int i)
{
	int mins, hours, days;
	char tmp2[30];

	ndbuff[0] = '\0';
	if(i >= 60)
	{
		mins = i / 60;
		i -= 60 * mins;
	}
	else mins = 0;
	if(mins >= 60)
	{
		hours = mins / 60;
		mins -= 60 * hours;
	}
	else hours = 0;
	if(hours >= 24)
	{
		days = hours / 24;
		hours -= 24 * days;
	}
	else days = 0;
	strcpy(ndbuff, "");

	if(days) {
	  sprintf(tmp2, "\002%d\002 jours", days);
	  strcat(ndbuff, tmp2);
	}
	if(hours) {
	  if(days) strcat(ndbuff, " ");
	  sprintf(tmp2, "\002%d\002 heures", hours);
	  strcat(ndbuff, tmp2);
	}
	if(mins) {
	  if(days||hours) strcat(ndbuff, " ");
	  sprintf(tmp2, "\002%d\002 minutes", mins);
	  strcat(ndbuff, tmp2);
	}
	if(!((mins || hours || days) && !i)) {
	  if(days||hours||mins) strcat(ndbuff, " et ");
	  sprintf(tmp2, "\002%d\002 secondes", i);
	  strcat(ndbuff, tmp2);
	}
	return ndbuff;
}

char *get_time(long ttime)
{
  register struct tm *lt, *gm;
  struct tm gmbuf;
  time_t clock;

  tbuff[0]='\0';

  clock = ttime;
  gm = gmtime(&clock);
  memcpy(&gmbuf, gm, sizeof(gmbuf));
  gm = &gmbuf;
  lt = localtime(&clock);

  snprintf(tbuff, 70, "%s %d %s %d %02d:%02d:%02d",
      weekdays[lt->tm_wday], lt->tm_mday, months[lt->tm_mon],
      1900 + lt->tm_year, lt->tm_hour, lt->tm_min, lt->tm_sec);

  return tbuff;

}

char *bottime()
{
   time_t timer;
   struct tm *ntime;
   char *buf;
   buf = malloc(70);

   time(&timer);
   ntime=localtime(&timer);
   snprintf(buf, 70,"%02d:%02d:%02d",ntime->tm_hour,ntime->tm_min,ntime->tm_sec);
   return buf;
}

void putlog(char *fichier, const char *pattern, ...)
{
  char buf[512];
  va_list vl;

  if(fichier == 0 || fichier == '\0') return;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  log(fichier, "[%s] %s", get_time(time(NULL)), buf);
}

void putserv(const char *pattern, ...)
{
  char buf[700];
  int len;
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 700, pattern, vl);
  va_end(vl);
  len = strlen(buf);
  send(bot.sock, buf, len,0);
  dataS += len;
  if(!strstr(buf, " PING ") && !strstr(buf, " PONG ")) putlog(LOG_PARSES, "S - %s",buf);
#ifdef DEBUG
  printf("SEND: %s", buf);
#endif
}

void uwntc(char *num, const char *pattern, ...)
{
  char buf[512];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv(":%s NOTICE %s :%s\n", uw.nick, num, buf);
  tidle = time(NULL);
}

void putchan(const char *pattern, ...)
{
  char buf[512], *t;
  va_list vl;
  buf[0]='\0';
  t=bottime();
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv(":%s PRIVMSG %s :[%s] %s\n", uw.nick, uw.chan, t, buf);
  free(t);
  return;
}

void uwmap()
{
	free_maplist();
	end_map = 0;
	putserv(":%s MAP\n", uw.nick);
	return;
}

void uwmode(const char *chan, const char *pattern, ...)
{
  char buf[512];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv(":%s MODE %s %s\n", bot.server, chan, buf);
  putchan("\2MODE\2 %s %s par %s", chan, buf, bot.server);
}

void uwtopic(const char *chan, const char *pattern, ...)
{
  char buf[TOPICLEN];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, TOPICLEN, pattern, vl);
  va_end(vl);
  putserv(":%s TOPIC %s :%s\n", uw.nick, chan, buf);
}

void uwkick(const char *chan, const char *vict, const char *pattern, ...)
{
  char buf[512];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv(":%s KICK %s %s :%s\n", bot.server, chan, vict, buf);
  putchan("\2KICK\2 %s %s par %s (%s)", chan, vict, bot.server, buf);
}

void uwwallops(const char *pattern, ...)
{
  char buf[300];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 300, pattern, vl);
  va_end(vl);
  putserv(":%s WALLOPS :%s\n", uw.nick, buf);
}

void uwpart(const char *chan, const char *pattern, ...)
{
	char buf[300];
	va_list vl;
	buf[0]='\0';
	va_start(vl, pattern);
	vsnprintf(buf, 300, pattern, vl);
	va_end(vl);
	putserv(":%s PART %s :%s\n", uw.nick, chan, buf);
	if(buf)
		putchan("\2PART\2 %s par %s (%s)", chan, uw.nick, buf);
	else
		putchan("\2PART\2 %s par %s", chan, uw.nick);
}

char *make_servnum(int c)
{
  int i;
  static char res[3];

  memset(res, 0, 3);
  for (i=0; i>=0; c = c >> 6, i--)
  {
    res[i] = base64[c & 63];
  }
  return res;
}

int resolve(char *host)
{
  struct hostent *he;
  he=gethostbyname(host);
  if(he==NULL) return(inet_addr(host));
  return (*(int *)he->h_addr);
}

char *strip_lastword(char *rest)
{
   char temp[500], *tmp2 = NULL, *tmp3, *strbuff;
   int nbmots = 1, i = 1;

   strbuff = malloc(500);
   strbuff[0] ='\0';
   temp[0] ='\0';
   strncpy(temp, rest, 500);
   strcpy(tmp2,rest);
   while (*tmp2 != '\0')
   {
      //send_to_server(":%s NOTICE #kouak :nbmots : %d\n", bot.nick, nbmots);
      if(*tmp2 == ' ') nbmots++;
      if(!strchr(tmp2, ' ')) break;
	  tmp2++;
   }
   if(nbmots==1) strncat(strbuff, "********", 500);
   else
   {
       tmp3 = strtok(temp, " ");
       strncpy(strbuff, tmp3, 500);
       i++;
   }
   //send_to_server(":%s NOTICE #kouak :nbmots : %s\n", bot.nick, toreturn);
   while(i < nbmots)
   {
      tmp3 = strtok(NULL, " ");
	  snprintf(strbuff, 500, "%s %s", strbuff, tmp3);
	  //send_to_server(":%s NOTICE #kouak :nbmots : %s\n", bot.nick, toreturn);
	  i++;
   }
   if(nbmots != 1) snprintf(strbuff, 500, "%s ********", strbuff);
   strbuff[strlen(strbuff) + 1] ='\0';
   return strbuff;
}


int getgloballevel(aNick *nick)
{
   return !nick->user ? 0 : nick->user->level;
}

/*char *strupr(char *buf)
{
   int i = 0;
   char tbuff, nbuff;

   ndbuff[0]='\0';
   while(buf[i] != '\0')
   {
      if(islower(buf[i]))
      {
		  tbuff = toupper(buf[i]);
		  ndbuff[i++] = tbuff;
      }
      else if(isspace(buf[i])) ndbuff[i++] = ' ';
      else
      {
		  nbuff = buf[i-1];
		  ndbuff[i++] = nbuff;
      }
   }
   ndbuff[i] = '\0';
   return ndbuff;
}*/
/*void wsntc(char *num, const char *pattern, ...)
{
  char buf[512];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv(":%s NOTICE %s :%s\n", ws.nick, num, buf);
  tidle = time(NULL);
}*/
/*struct access *getaccesinfo(char *nick, char *chan)
{
   anUser *current;
   anAccess *access;

   current = userhead;
   while(current)
   {
      if(!strcasecmp(nick,current->nick))
      {
         for(access = current->accesshead;access && strcasecmp(chan, access->chan);access = access->next);
		 return access;
      }
      current = current->next;
   }
   return NULL;
}*/
/*int IsBadChan(char *chan)
{
   struct badchans *current;
   current = badchead;
   while(current)
   {
      if(!match(current->chaine, chan)) return 1;
      current = current->next;
   }
   return 0;
}*/
/*char *convert_addr(char *who, char *address)
{
   struct nickinfo *nick;
   char *hostip = NULL, *ptr2, backup[1000], *addr;
   backup[0]='\0';
   if(!is_ip(address))
   {
      if((addr = strchr(address, '@'))) *addr++ = '\0';
      nick = nickhead;
      while(nick)
      {
         if(!strcasecmp(who, nick->nick)) hostip = nick->host;
         nick = nick->next;
      }
      strcpy(strbuff, hostip);
      if((ptr2 = strrchr(strbuff, '.'))) *++ptr2 = '*';
      *++ptr2 = '\0';
      if(strlen(address) > 8)
      {
         address[8] = '*';
         address[9] = '\0';
      }
      sprintf(backup, "%s@%s", address, strbuff);
      strcpy(strbuff, backup);
      return strbuff;
   }
   return NULL;
}*/
/*char *UserNick(aNick *nick) //FOLIE DE PROGS : malloc sans free ...
{
	char *thenick;

	thenick = malloc(200);
	if(nick->user)
		snprintf(thenick, 200, "%s@%s", nick->nick, nick->user->nick);
	else
		strncpy(thenick, nick->nick, NICKLEN);
	return thenick;
}*/
/*void trailing_blanks(char *str)
{
   int i = 0;
   for(i = strlen(str); i > 0; i--)
   {
      if(isspace(str[i-1])) str[i-1] = '\0';
      else return;
   }
}*/
